'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

const blessed = require('blessed');

const LABEL_APP = 'UTop ver.';
const LABEL_PID = 'PID: ';

class Header {
  constructor(_ref) {
    let screen = _ref.screen,
        command = _ref.command,
        version = _ref.version;
    var _ref$pid = _ref.pid;
    let pid = _ref$pid === undefined ? '-' : _ref$pid;
    var _ref$time = _ref.time;

    let time = _ref$time === undefined ? '--:--:--' : _ref$time,
        props = _objectWithoutProperties(_ref, ['screen', 'command', 'version', 'pid', 'time']);

    this.screen = screen;
    this.command = command;
    this.version = version;
    this.pid = pid;
    this.time = time;
    this.props = props;

    this._render();
  }

  _render() {
    let title = blessed.escape(this.command);
    const limit = this.screen.width - LABEL_PID.length - String(this.pid).length - String(this.time).length - LABEL_APP.length - String(this.version).length - 9;

    if (title.length > limit) {
      title = `${title.substr(0, limit - 3)}...`;
    }

    const content = [`{bold}${title}{/bold}`, '{|}', `[${LABEL_PID}{bold}${this.pid}{/bold}] `, `[{bold}${this.time}{/bold}] `, `${LABEL_APP}${this.version}`].join('');

    if (this._chart) {
      this._chart.setContent(content);
      this.screen.render();
    } else {
      this._chart = blessed.text(_extends({
        parent: this.screen,
        tags: true,
        content
      }, this.props));
    }
  }

  update({ pid, time }) {
    this.pid = pid || this.pid;
    this.time = time || this.time;
    this._render();
  }
}

module.exports = Header;