'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

const blessed = require('blessed');
const sparkline = require('sparkline');

class SparklineChart {
  constructor(_ref) {
    let title = _ref.title;
    var _ref$maxValue = _ref.maxValue;
    let maxValue = _ref$maxValue === undefined ? 0 : _ref$maxValue;
    var _ref$valuePadding = _ref.valuePadding;
    let valuePadding = _ref$valuePadding === undefined ? 6 : _ref$valuePadding;
    var _ref$colorOk = _ref.colorOk;
    let colorOk = _ref$colorOk === undefined ? 'yellow' : _ref$colorOk;
    var _ref$colorWarn = _ref.colorWarn;

    let colorWarn = _ref$colorWarn === undefined ? 'red' : _ref$colorWarn,
        screen = _ref.screen,
        props = _objectWithoutProperties(_ref, ['title', 'maxValue', 'valuePadding', 'colorOk', 'colorWarn', 'screen']);

    this.title = title;
    this.maxValue = maxValue;
    this.valuePadding = valuePadding;
    this.colorOk = colorOk;
    this.colorWarn = colorWarn;
    this.screen = screen;
    this.props = props;

    this._dataArray = [];

    this._init();
    this._render();

    this.screen.on('resize', () => {
      this._init();
      this._render();
    });
  }

  _init() {
    const countLimit = Math.max(this.screen.width - this.title.length - this.valuePadding - 1, 1);

    let oldDataArray = this._dataArray;
    if (oldDataArray.length > countLimit) {
      oldDataArray = this._dataArray.slice(-countLimit);
    }

    this._dataArray = new Array(countLimit - oldDataArray.length).fill(0).concat(oldDataArray);
  }

  _render(value = 0, printValue = '', postfix = '') {
    // first one is allways 100% for the right 0-maxValue scale
    const graphString = sparkline([this.maxValue].concat(this._dataArray.slice(1))).split('').map((s, i) => {
      if (i === 0) {
        return ''; // do transparent first one fake value
      } else if (this.maxValue && this._dataArray[i] > this.maxValue * 0.66) {
        return `{${this.colorWarn}-fg}${s}{/${this.colorWarn}-fg}`;
      } else {
        return `{${this.colorOk}-fg}${s}{/${this.colorOk}-fg}`;
      }
    }).join('');

    const content = `${this.title} ${graphString} {bold}${printValue}{/bold}${postfix}`;

    if (this._chart) {
      this._chart.setContent(content);
      this.screen.render();
    } else {
      this._chart = blessed.text(_extends({
        parent: this.screen,
        tags: true,
        content
      }, this.props));
    }
  }

  add(rawValue, printValue = '', postfix = '') {
    let validatedValue = Math.ceil(rawValue);
    if (this.maxValue) {
      validatedValue = Math.min(validatedValue, this.maxValue);
    }

    this._dataArray.push(validatedValue);
    this._dataArray.shift();
    this._render(validatedValue, printValue, postfix);
  }
}

module.exports = SparklineChart;